;                         JPMOUSE.asm

;       Program to deal with mouse in dBASE IV Version 1.5
;       Jay Parsons, BORBBS Jparsons, CompuServe 70160,340
;       and Ken Chan (BORBBS HazMatZak)
;       Thanks to Dave Frankenbach (Frnknbch) and Bowen Moursund (Bowen)
;       Version 2.01, July 8, 1992
;       see README.txt for complete description

;       argument:   'R' to check for and reset mouse and turn it on.
;                   '?' to check for and reset mouse, but NOT turn it on.
;                   '+' to enable driver and show mouse cursor
;                   '-' to disable driver and hide mouse cursor
;                   'H' to hide cursor only
;                   'S' to show cursor only
;                   any argument below chr(43), or chr(255):
;                       first argument is interpreted as row, second as column,
;                       both as chr() values.  If argument is chr(255),
;                       mouse is not moved, otherwise is is moved to that
;                       row, column or both.  In any case the mouse cursor
;                       row and column are returned.  If only one argument
;                       is furnished, mouse will be moved to column zero.

;       returns:    If called with '?' or 'R' returns 'T' if mouse, or 'F'.
;                   Calls with first argument below chr(43), or chr(255),
;                   return row and column of mouse cursor as chr() values.
;                   Calls with first argument $ "+-HS" return 'T',
;                   call with some other character returns 'F'.

                .MODEL tiny
                .CODE

                int 3                           ; for debugging
                jcxz done                       ; if no argument, quit
                mov ah,ch                       ; zero ah for mouse calls
                lds si,es:[di]                  ; point to argument 1
                push si                         ; save its address
                push ds
                lodsb                           ; fetch it
                cmp al,255                      ; report row?
                je getpos                       ; if so, jump to it
                cmp al,'A'                      ; alphabetical?
                jb notalpha                     ; if not, jump
                and al,5Fh                      ; if so, capitalize
                cmp al,'H'
                jz hide                         ; if 'H', just hide cursor
                cmp al,'S'
                jz show                         ; if 'S', show it
                cmp al,'R'
                jnz nomouse                     ; if not 'R', bail out
                jmp short reset

notalpha:       cmp al,43                       ; "+"?
                jb getpos                       ; lower is change row
                cmp al,'?'                      ; if '?', test and reset only
                jz reset
                cmp al,'-'
                jz off                          ; if '-', off
                cmp al,'+'
                jnz nomouse                     ; else if not '+', quit

on:             mov al,20h                      ; enable driver
                int 33h
show:           mov ax,1                        ; Show cursor
                int 33h
                jmp short gotmouse

off:            mov al,1Fh                      ; disable
                int 33h
hide:           mov ax,2                        ; hide cursor
                int 33h

gotmouse:       mov al,'T'
rmouse:         pop es
                pop di                          ; store first argument
                stosb
done:           retf

getpos:         lds si,es:[di+4]                ; get column
                mov ah,[si]                     ; to ah
                push ax                         ; save row and column arguments
                mov ax,3                        ; get mouse position
                int 33h
                pop ax                          ; retrieve arguments
                mov bx,cx                       ; save horizontal position
                mov cx,3                        ; set up for shifts and ch=0
                cmp ah,255                      ; was a column change ordered?
                jnz colmove                     ; if nonzero, al is ok
                sar bx,cl                       ; divide by 8 for column
                mov ah,bl                       ; save column
                mov [si],ah                     ; and argument for return

colmove:        cmp al,255                      ; row change ordered?
                jnz rowmove
                sar dx,cl                       ; divide by 8 for row
                mov al,dl                       ; save row

rowmove:        mov bx,ax                       ; save row for return
                mov dh,ch                       ; zero out dh
                mov dl,al                       ; get row
                sal dx,cl                       ; convert to pixels
                mov al,ah                       ; put column in al
                mov ah,ch                       ; now in ax
                sal ax,cl                       ; convert to pixels
                mov cx,ax                       ; and put here for move
                mov ax,4                        ; set mouse position
                int 33h
                mov al,bl                       ; put row in al
                jmp short rmouse                ; and return it

reset:          mov si,ax                       ; save argument
                mov ax,03533h                   ; Get Int 33H vector
                int 21h                         ; by calling Int 21H.
                mov ax,es                       ; Check segment and offset of
                or ax,bx                        ; Int 33H.  If 0 then driver
                jz nomouse                      ; is not installed.
                cmp byte ptr es:[bx],0CFh       ; Also, if IRET then driver is
                jnz init                        ; not installed.
nomouse:        mov al,'F'
                jmp short rmouse

init:           xor ax,ax                       ; Initialize mouse
                int 33h
                or ax,ax                        ; Is mouse installed?
                jz nomouse                      ; if no mouse, return
                mov ax,si                       ; get argument back
                cmp al,'R'
                jnz gotmouse                    ; if not 'R', don't show it
                jmp short on

                end
